//
// BinaryFileIndex.h - types for binary index file layout
//

#ifndef inc_BinaryFileIndex_h
#define inc_BinaryFileIndex_h

#include "vos\Errors.h"
#include "vos\Time.h"

//
// TIndexEntryType - specifies the type of an index entry.
//

typedef enum
{
	IndexEntryType_File,
	IndexEntryType_Folder
}
TIndexEntryType;

//
// Value to indicate root name with no prefix segment.
//
enum { IndexPrefix_None = 0xFFFFFFFF };

//
// TIndexStatus - specifies whether an index was built successfully or not.
// (The index is marked as bad when it is created, and then marked as good
//  when it has been completely created).
//
typedef enum
{
	IndexStatus_Good,
	IndexStatus_Bad
}
TIndexStatus;

//
// Current version of the binary file index.
//
#define kBinaryIndexVersion (1)

/**************************************************************************

	class BinaryIndexHeader

	Author:		Tim Browse

	Date:		2001/10/24

	Purpose:	Header structure stored at the start of a FastFind .idx 
				file (when it's a binary index).

				It includes simple information:

					- Version number of index format
					- Status of the index (good or bad?)
					- Number of entries in the index.

**************************************************************************/

class BinaryIndexHeader
{
	public:
		BinaryIndexHeader() : m_Version(kBinaryIndexVersion), 
							  m_Status(IndexStatus_Bad),
							  m_cEntries(0)
		{
			// TCILB
		}

	public:
		// Version number of index
		int m_Version;

		// Current status of index
		TIndexStatus m_Status;

		// Number of entries in the index
		int m_cEntries;
};

/**************************************************************************

	class BinaryIndexEntry

	Author:		Tim Browse

	Date:		2001/10/24

	Purpose:	Hold information about an entry in the index.  The actual
				leafname is not held directly in the structure - if the
				entry has been loaded from disk, the leafname is stored after
				the object data.

				When writing the index, the leafname is passed in directly
				so it can be written to the index - see WriteEntry().

**************************************************************************/

#include "vOS\StructPackingOn.h"

class BinaryIndexEntry
{
	public:
		// Accessors
		uint32 GetFileSize() const;
		vos::Time GetTime() const;

		void SetFileSize(uint32 FileSize);
		void SetTime(vos::Time Time);

	public:
		// Type of this entry
		uint8	m_Type;

		// Length of the name of the entry
		uint8 m_NameLength;

		// Prefix of this entry
		uint32 m_Prefix;

	public:
		// Write this entry to a file
		vos::Error WriteEntry(vos::File* pFile, const char* szLeafname);

		// Access raw string in this entry (which is actually after this entry)
		// NB. Only available when index entry has been loaded from disk;
		//     not when writing the index.
		const char* GetName() const;

	private:
		// Size in bytes (if a file)
		uint32 m_cbFileSize;

		// Timestamp of the file/folder
		vos::Time m_Time;
};

#include "vOS\StructPackingOff.h"


#endif // !inc_BinaryFileIndex_h
