//
// FileIndexBuilder.cpp -- class to build an index
//

#include "StdHdr.h"

#include "FileIndexBuilder.h"

#include "vos\Errors.h"
#include "vos\FileInfo.h"

#include "FileIndex.h"
#include "FileIndexWriter.h"

/**************************************************************************
								   
>	FileIndexBuilder::FileIndexBuilder(FileIndexWriter* pIndex,
									   const vos::FileSpec& FolderToIndex,
									   FileIndexBuilder* pParent)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		pIndex - the index writer to use to generate the index.
				FolderToIndex - the folder we should start indexing from.
				pParent - the parent index builder for this object (NULL if
						  the builder is the scanning the root folder).

	Purpose:	Constructor

**************************************************************************/

FileIndexBuilder::FileIndexBuilder(FileIndexWriter* pIndex,
								   const vos::FileSpec& FolderToIndex,
								   FileIndexBuilder* pParent) : m_pIndex(pIndex),
																m_pParent(pParent),
																m_Folder(FolderToIndex),
																m_pFiles(NULL),
																m_pChild(NULL)
{
	// Add this folder to the index
	vos::Time Time(0);
	if (!FolderToIndex.GetFileTime(&Time))
	{
		// Error getting file time - plough on regardless
		ERROR_HANDLED();
	}

	m_pIndex->AddFolder(FolderToIndex, Time);

	// Create the file matching object
	vos::FileSpec FilesToFind = m_Folder;
	FilesToFind.AppendFilename("*.*");

	// Search for all files/folders contained in this folder.
	m_pFiles = new vos::FileMatches(FilesToFind);
}

/**************************************************************************

>	FileIndexBuilder::~FileIndexBuilder()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Destructor

**************************************************************************/

FileIndexBuilder::~FileIndexBuilder()
{
	delete m_pFiles;
	delete m_pChild;
}

/**************************************************************************

>	vos::Error FileIndexBuilder::BuildIndex(bool bFileLimit,
											int* pcFilesToIndex,
											bool* pbFinished)


	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		bFileLimit - true => limit the number of files to be indexed.
							 false => keep going until all files have been indexed.

				pcFilesToIndex - if bFileLimit is true, then this specifies
								 the number of files that should be indexed
								 before returning control to the caller.

	Outputs:	pcFilesToIndex - the number of files left that can be indexed;
								 this is either 0, which means the limit was
								 reached, or is non-zero, which means we ran out
								 of files to index in this folder.

				pbFinished - true => we have finished indexing files in this
									 folder.
							 false => there are more files to index, but we
									  hit the file limit, so we had to stop.
	Returns:	Error

	Purpose:	Add some more files to the index.

				This is a recursive function, and also an interuptible one;
				the caller can specify that files are indexed in blocks of
				N files at a time.  This is to allow a responsive GUI 
				to be provided, without all that tedious mucking about
				with multi-threading.

**************************************************************************/

vos::Error FileIndexBuilder::BuildIndex(bool bFileLimit,
										int* pcFilesToIndex,
										bool* pbFinished)
{
	COMPLAIN_IF(pbFinished == NULL, "Illegal NULL pointer in BuildIndex");

	// Service any active child index builder first
	if (m_pChild != NULL)
	{
		vos::Error Result = m_pChild->BuildIndex(bFileLimit, pcFilesToIndex, pbFinished);

		if (*pbFinished)
		{
			// Child indexer finished, so delete it
			delete m_pChild;
			m_pChild = NULL;
		}
		else
		{
			// Not finished; we're timeslicing - return to caller
			return Result;
		}
	}

	// No child indexers active - so carry on with this folder.
	vos::FileSpec FileName;
	vos::FileInfo FileInfo;

	// Keep going if there's no limit on files to index, or there is
	// a limit, but we haven't reached it yet.
	while (!bFileLimit || ((*pcFilesToIndex) > 0))
	{
		// Do next file
		if (m_pFiles->GetNextFile(&FileName, &FileInfo))
		{
			// If this is a folder, start a child index to index it
			if (FileInfo.m_bIsFolder)
			{
				// Create the new indexer - it will add the folder to the index...
				m_pChild = new FileIndexBuilder(m_pIndex, FileName, this);

				vos::Error Result = m_pChild->BuildIndex(bFileLimit, pcFilesToIndex, pbFinished);

				if (*pbFinished)
				{
					// Child indexer finished, so delete it
					delete m_pChild;
					m_pChild = NULL;
				}

				// If an error occurred, stop indexing.
				if (!Result)
				{
					return LASTERROR;
				}
			}
			else
			{
				// Just add the filename to the index
				m_pIndex->AddFile(FileName, 
								  FileInfo.m_Size,
								  FileInfo.m_Modified);

				// If caller has placed a limit on our activity, decrement
				// the file counter so that we stop when we are supposed to.
				if (bFileLimit)
				{
					(*pcFilesToIndex)--;
				}
			}
		}
		else
		{
			// No more files - we're finished
			*pbFinished = true;
			return VNOERROR;
		}
	}

	// Finished this batch of files (but not the whole index)
	*pbFinished = false;
	return VNOERROR;
}

/**************************************************************************

>	vos::Error FileIndexBuilder::BuildIndex()

	Author:		Tim Browse
	Date:		2001/10/25
	Returns:	Error

	Purpose:	Build the whole index in one go, with no timeslicing.
				
				This function will NOT return until the whole index has been
				built, which can be of the order of minutes.

**************************************************************************/

vos::Error FileIndexBuilder::BuildIndex()
{
	bool bFinished;
	return BuildIndex(false, 0, &bFinished);
}



/**************************************************************************
									    
>	vos::Error FileIndexBuilder::BuildIndex(int cFilesToIndex,
										    bool* pbFinished = NULL)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		cFilesToIndex - the number of files/folders to add to
								the index before returning.
	Outputs:	pbFinished - true -> index is finished;
							 false => there are still files left to add to 
									  the index.
	Returns:	Error

	Purpose:	Friendly publically accessible veneer for the 
				more complex recursive version of BuildIndex().

**************************************************************************/

vos::Error FileIndexBuilder::BuildIndex(int cFilesToIndex,
									    bool* pbFinished)
{
	// If they do not want to know if the index is finished (then they're
	// mad!), then we provide a dummy variable for the recursive function
	// to use.
	bool bFinished;
	if (pbFinished == NULL)
		pbFinished = &bFinished;

	return BuildIndex(true, &cFilesToIndex, pbFinished);
}


/**************************************************************************

>	void FileIndexBuilder::GetCurrentFolder(vos::FileSpec* pCurrentFolder)

	Author:		Tim Browse
	Date:		2001/10/25
	Outputs:	pCurrentFolder - the folder we are currently indexing.

	Purpose:	Find out which folder we are currently at in the index
				building process.

**************************************************************************/

void FileIndexBuilder::GetCurrentFolder(vos::FileSpec* pCurrentFolder)
{
	// Recursive - if the child is active, ask it for the current folder...
	if (m_pChild != NULL)
	{
		// Defer to child
		m_pChild->GetCurrentFolder(pCurrentFolder);
	}
	else
	{
		// Otherwise, return our folder.
		*pCurrentFolder = m_Folder;
	}
}

