//
// FileMover.cpp
//

#include "StdHdr.h"

#include "FileMover.h"

#include "vos\FileSpec.h"
#include "vos\VolumeSpec.h"
#include "vos\Errors.h"

#include "vgui\Alert.h"

#include "Res\Resource.h"

#include "ProgressDialog.h"

/**************************************************************************
					 
>	FileMover::FileMover(const vos::FileSpec& fsSource,
						 const vos::FileSpec& fsDest)

	Author:		Tim Browse
	Date:		2001/10/27
	Inputs:		fsSource - the source file to move
				fsDest - where to move the file to.

	Purpose:	Move a file from one place to another, displaying a progress
				dialog during the move operation.

				If the file is moved to a different volume, the file is
				copied, and then the original is deleted.

**************************************************************************/

FileMover::FileMover(const vos::FileSpec& SourceFile,
					 const vos::FileSpec& DestFile) : m_fsSource(SourceFile),
													  m_fsDest(DestFile),
													  m_ResultCode(FileMover_Success),
													  m_pDialog(NULL)
{
	// TCILB
}

/**************************************************************************

>	void FileMover::Prepare()

	Author:		Tim Browse
	Date:		2001/10/27

	Purpose:	Prepare to move the file.

**************************************************************************/

void FileMover::Prepare()
{
	// No preparation required...
}

/**************************************************************************

>	void FileMover::DoWork(ProgressDialog* pProgressDialog)

	Author:		Tim Browse
	Date:		2001/10/27
	Inputs:		pProgressDialog - the progress dialog to use to show
								  progress.

	Purpose:	Move the file, updating the progress display during the move.

**************************************************************************/

void FileMover::DoWork(ProgressDialog* pProgressDialog)
{
	// Remember the dialog pointer
	m_pDialog = pProgressDialog;

	// Check the destination file does not already exist...
	if (m_fsDest.GetFileType() != vos::FileType_NotFound)
	{
		// It does, so we can't move the file...
		m_ResultCode = FileMover_FileAlreadyExists;
		return;
	}
#if 0
	// Ok, first take the easy way out - try to rename the file...
	if (m_fsSource.Rename(m_fsDest))
	{
		// File moved successfully - yay!
		pProgressDialog->UpdateProgressDisplay(100);
		return;
	}
	else
	{
		// Unable to rename - ok, clear the error, and let's try
		// copy/delete instead.
		ERROR_HANDLED();
	}
#endif
	// Ok, we need to copy the file then delete the original...check that
	// we seem to have enough space before we start the copy...
	m_cbFileSize = m_fsSource.GetFileSize();

	vos::VolumeSpec vsDestVolume(m_fsDest);
	uint cBytesFree;
	if (vsDestVolume.GetFreeSpace(&cBytesFree))
	{
		if (cBytesFree < m_cbFileSize)
		{
			// All done - we can't move the index.
			m_ResultCode = FileMover_NoSpace;
			return;
		}
	}
	else
	{
		// Unable to get free space - ignore and carry on anyway.
		ERROR_HANDLED();
	}

	// Ok, try to copy the index to the new place.
	if (m_fsSource.Copy(m_fsDest, vos::FileCreation_Normal, this))
	{
		// File was copied successfully - so delete the original.
		if (m_fsSource.Delete())
		{
			// Operation completed ok.
			pProgressDialog->UpdateProgressDisplay(100);
			return;
		}
		else
		{
			// Unable to delete original file - restore the status quo by 
			// deleting the new copy.

			// Remember error text (so it can be displayed to user if necessary)
			m_ErrorMsg = vos::GetLastError()->GetText();

			ERROR_HANDLED();

			if (!m_fsDest.Delete())
			{
				ERROR_HANDLED();
			}

			m_ResultCode = FileMover_DeleteFailed;
		}
	}
	else
	{
		// Failed to copy the index file.

		// Remember error text (so it can be displayed to user if necessary)
		m_ErrorMsg = vos::GetLastError()->GetText();

		ERROR_HANDLED();

		// If the copy failed for any reason other than the user cancelling it,
		// flag the error to the caller.
		if (m_ResultCode != FileMover_UserCancelled)
			m_ResultCode = FileMover_CopyFailed;
	}
}

/**************************************************************************

>	vos::Error FileMover::CopyFileCallback(uint cBytesCopied)

	Author:		Tim Browse
	Date:		2001/10/27
	Inputs:		-
	Outputs:	-
	Returns:	-

	Purpose:	Prove that Tim is a jerk!

**************************************************************************/

vos::Error FileMover::CopyFileCallback(uint cBytesCopied)
{
	// Update the progress display with percentage of file copied,
	// and check for cancellation of operation.
	if (m_pDialog != NULL)
	{
		m_pDialog->UpdateProgressDisplay((cBytesCopied * 100) / m_cbFileSize);

		if (m_pDialog->Cancelled())
		{
			// User has chosen to cancel file copy, so just return an error
			// to the caller to signify that it should stop copying.
			m_ResultCode = FileMover_UserCancelled;
			return ERROR(Err_GenericError, "User cancelled operation.");
		}
	}
	else
	{
		COMPLAIN("No dialog pointer in CopyFileCallback()!");
	}

	return VNOERROR;
}

/**************************************************************************

>	bool FileMover::GetErrorText(vos::String* pErrorMsg)

	Author:		Tim Browse
	Date:		2001/10/28
	Outputs:	pErrorMsg - the text of the error, if available.
	Returns:	true => error message is available in pErrorMsg;
				false => no specific error message is available.

	Purpose:	Get the error message related to any error that has
				occurred, if it is available.

**************************************************************************/

bool FileMover::GetErrorText(vos::String* pErrorMsg)
{
	if (!m_ErrorMsg.IsEmpty())
	{
		// Return the error text to the caller
		*pErrorMsg = m_ErrorMsg;
		return true;
	}
	else
	{
		// No error message available
		return false;
	}
}
