//
// IndexSearcher.cpp
//

#ifndef inc_IndexSearcher_h
#define inc_IndexSearcher_h

#include "StdHdr.h"

#include "IndexSearcher.h"

IndexSearcher::IndexSearcher() : m_pIndex(NULL)
{
}

bool IndexSearcher::Init(FileIndex* pIndex,
						 vos::String strPattern, 
						 bool bUseRegExp,
						 bool bAutoWildcard)
{
	// Remember index object that we will search
	m_pIndex = pIndex;

	// Convert string to the regular expression we want to use
	vos::String strRegExp;
	
	if (bUseRegExp)
	{
		// Pattern is actually a complete regular expression, so use it as is.
		strRegExp = strPattern;
	}
	else
	{
		if (bAutoWildcard)
		{
			// If pattern contains no wildcards, then we add a * to the
			// start and end, so it automatically does partial filename
			// matches.
			if (!strPattern.Find("*") && !strPattern.Find("?"))
			{
				strPattern = "*" + strPattern + "*";
			}
		}

		// Pattern is a wildcard pattern, so convert this to regular expression format.
		strRegExp = MakeRegularExpression(strPattern);
	}

	try
	{
		// Ignore case when comparing
		RegExp.SetExpression(strRegExp.c_str(), true);
	}
	catch (...)
	{
		// Regular expression contains an error.
		return false;
	}

	// Get ready to do search
	m_pIndex->ClearSearchResults();
	m_iEntry = 0;

	// Ok
	return true;
}

vgui::TJobStatus IndexSearcher::DoWork(vgui::JobController* pJobController,
									   int* pPercentComplete)
{
	vos::String EntryName;

	int cEntries = m_pIndex->GetNumberOfEntries();

	// The regexp library can throw expressions
	try
	{
		// Check a block of files
		int cFilesToCheck = 10000;

		BinaryIndexEntry** pEntries = m_pIndex->GetEntries();

		while ((cFilesToCheck > 0) && (m_iEntry < cEntries))
		{
			// Look for string match
			EntryName = pEntries[m_iEntry]->GetName();
			EntryName.MakeLower();

			if (RegExp.Match(EntryName.c_str()))
			{
				// Got a match - add it to the results;
				m_pIndex->AddSearchResult(pEntries[m_iEntry]);
			}

			// Try next file
			m_iEntry++;
			cFilesToCheck--;
		}
	}
	catch (...)
	{
		// Probably invalid regexp
		return vgui::Job_Finished;
	}

	// Fill in percentage
	*pPercentComplete = (100 * m_iEntry) / cEntries;

	// ok, has search completed?
	if (m_iEntry < cEntries)
		return vgui::Job_NotFinished;
	else
		return vgui::Job_Finished;
}

/**************************************************************************

>	vos::String IndexSearcher::MakeRegularExpression(const vos::String& Pattern)

	Author:		Tim Browse
	Date:		30/3/2002
	Inputs:		Pattern - the standard filename wildcard pattern
	Returns:	Regular expression version of 'Pattern'.

	Purpose:	Convert a standard wildcarded filename (using * and ?)
				into a regular expression suitable to be passed to
				boost::RegEx::SetExpression().

**************************************************************************/

vos::String IndexSearcher::MakeRegularExpression(const vos::String& Pattern)
{
	// Simple 'dumb' conversion character by character...
	//
	//  * is translated to .*
	//  ? is translated to .
	//  . is translated to \.
	//
	vos::String NewPattern;

	int PatternLength = Pattern.GetLength();

	for (int iPattern = 0; iPattern < PatternLength; iPattern++)
	{
		// Translate character
		vos::tchar Ch = Pattern[iPattern];

		switch (Ch)
		{
			case '*':	NewPattern += ".*";		break;
			case '?':	NewPattern += ".";		break;
			case '.':	NewPattern += "\\.";	break;
			default:	NewPattern += Ch;		break;
		}
	}

	return NewPattern;
}



