//
// ProgressDialog.cpp
//

#include "StdHdr.h"

#include "ProgressDialog.h"

#include "vgui\Metrics.h"
#include "vgui\Timer.h"

#include "vos\Trace.h"
#include "vos\Errors.h"

/**************************************************************************

>	ProgressDialog::ProgressDialog(ProgressWorker_Timesliced* pWorker, 
								   const vos::String& strProgressMessage,
								   vgui::Window* pParent = NULL)

	Author:		Tim Browse
	Date:		24/07/2001
	Inputs:		pWorker - the object doing the actual job which we
						  are displaying the progress UI for.
				strProgressMessage - the progress message to display in
									 the dialog.
				pParent - the parent window of this dialog

	Purpose:	Constructor - for a progress dialog that repeatedly calls
				the worker object to carry out the operation in a timesliced
				manner.

**************************************************************************/

ProgressDialog::ProgressDialog(ProgressWorker_Timesliced* pWorker, 
							   const vos::String& strProgressMessage,
							   vgui::Window* pParent) : vgui::Dialog(vgui::DialogResourceID::s_DynamicDialog, pParent),
														m_strProgressMessage(strProgressMessage),
														m_pTimeslicedWorker(pWorker),
														m_pOneShotWorker(NULL),
														m_bCancelled(false),
														m_TimerID(vgui::NullTimer)
{
	// TCILB
}


/**************************************************************************

>	ProgressDialog::ProgressDialog(ProgressWorker_OneShot* pWorker, 
								   const vos::String& strProgressMessage,
								   vgui::Window* pParent = NULL)

	Author:		Tim Browse
	Date:		24/07/2001
	Inputs:		pWorker - the object doing the actual job which we
						  are displaying the progress UI for.
				strProgressMessage - the progress message to display in
									 the dialog.
				pParent - the parent window of this dialog

	Purpose:	Constructor - for a progress dialog that hands over control
				to a one-shot progress worker to do the job in one go.

**************************************************************************/

ProgressDialog::ProgressDialog(ProgressWorker_OneShot* pWorker, 
							   const vos::String& strProgressMessage,
							   vgui::Window* pParent) : vgui::Dialog(vgui::DialogResourceID::s_DynamicDialog, pParent),
														m_strProgressMessage(strProgressMessage),
														m_pTimeslicedWorker(NULL),
														m_pOneShotWorker(pWorker),
														m_bCancelled(false),
														m_TimerID(vgui::NullTimer)
{
	// TCILB
}




/**************************************************************************

>	void ProgressDialog::OnInitDialog()

	Author:		Tim Browse
	Date:		25/7/2001

	Purpose:	Set up the progress bar etc in the dialog.

**************************************************************************/

void ProgressDialog::OnInitDialog()
{
	// Set the title text
	SetText(m_strProgressMessage);

	// Layout metrics
	const int kGap = 6;
	const int kDialogWidth = 300;
	const int kButtonWidth = vgui::Metrics::GetButtonWidth(this);
	const int kButtonHeight = vgui::Metrics::GetButtonHeight(this);
	const int kProgressBarHeight = vgui::Metrics::GetProgressBarHeight(this);

	// Create progress bar control
	vgui::Rect ProgressPosition(kGap, kGap, kDialogWidth - kGap, kGap + kProgressBarHeight);
	m_ProgressBar.Create(this, ProgressPosition, vgui::NOCTL);

	// Control is always a percentage - 0% to 100%
	m_ProgressBar.SetRange(0, 100);

	// Create 'cancel' button
	vgui::Rect CancelPosition;
	CancelPosition.left = (kDialogWidth / 2) - (kButtonWidth / 2);
	CancelPosition.top  = kGap + kProgressBarHeight + kGap;
	CancelPosition.right = CancelPosition.left + kButtonWidth;
	CancelPosition.bottom = CancelPosition.top + kButtonHeight;
	
	m_Cancel.Create(this, CancelPosition, IDCANCEL);
	m_Cancel.SetText("Cancel");

	// Resize dialog to desired size
	vgui::Rect NewClientArea(0,0, 
							 kDialogWidth, 
							 kGap + kProgressBarHeight + kGap + kButtonHeight + kGap);
	SetClientArea(NewClientArea);

	if (m_pTimeslicedWorker != NULL)
	{
		// Set timer callback periodically (every 50ms) so we can do chunks of work
		m_TimerID = vgui::Timer::SetTimer(vgui::TimerType_Periodic, 50, this);
	}
	else
	{
		// Set timer callback once so we can trigger the job to start.
		m_TimerID = vgui::Timer::SetTimer(vgui::TimerType_OneShot, 0, this);
	}

	// Tell worker object to get ready
	if (m_pTimeslicedWorker != NULL)
		m_pTimeslicedWorker->Prepare();
	else
		m_pOneShotWorker->Prepare();
}

/**************************************************************************

>	bool ProgressDialog::OnTimer(vgui::TimerID ID)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		ID - ID of the timer we are processing
	Returns:	true (always)

	Purpose:	Handler timer events - this is when the actual 'work' 
				gets done - we call the worker object to do a bit of work.

**************************************************************************/

bool ProgressDialog::OnTimer(vgui::TimerID ID)
{
	// If user's super quick and has already cancelled, then check out of here!
	if (m_bCancelled)
	{
		EndDialog(IDOK);
		return true;
	}

	// Sanity check
	COMPLAIN_IF(ID != m_TimerID, 
				"ProgressDialog::OnTimer(): Wrong timer ID in timer event!");

	if (m_pTimeslicedWorker == NULL)
	{
		// One-shot type of operation - do the work
		m_pOneShotWorker->DoWork(this);

		// All done, so get rid of this dialog.
		EndDialog(IDOK);
		return true;
	}
	else
	{
		// Periodic operatin - ok, let's call our worker object to see 
		// if it wants to do any more work.
		int PercentComplete;

		if (m_pTimeslicedWorker->DoWork(&PercentComplete) == ProgressWorker_Finished)
		{
			// All done, so get rid of this dialog.
			EndDialog(IDOK);
			return true;
		}

		// Ok, not finished, so update the progress bar position
		SetPosition(PercentComplete);
	}

	return true;
}

/**************************************************************************

>	void ProgressDialog::OnCancel()

	Author:		Tim Browse
	Date:		2001/10/25

	Purpose:	Handle the user cancelling the operation - we tell the 
				worker object that the operation has been cancelled, and
				then get rid of this dialog.

**************************************************************************/

void ProgressDialog::OnCancel()
{
	// User has pressed cancel
	m_bCancelled = true;

	// If we're doing timesliced operation, shut this down...
	if (m_pTimeslicedWorker != NULL)
	{
		// First, kill our timer event so we don't get any more timer events.
		// NB. This is particularly important if the worker object displays
		//	   any dialogs during OnCancel().
		vgui::Timer::KillTimer(m_TimerID);

		// Tell the worker object that the user has cancelled
		m_pTimeslicedWorker->OnCancel(this);

		// And shut this dialog down using default behaviour
		Dialog::OnCancel();
	}
	else
	{
		// The one-shot worker object should be calling Cancelled()
		// periodically, and respond accordingly, so we do nothing here...
	}
}

/**************************************************************************

>	bool ProgressDialog::Cancelled() const

	Author:		Tim Browse
	Date:		2001/10/27
	Returns:	true => user has clicked the cancel button in the dialog.
				false => operation should continue.

	Purpose:	Find out if the user has cancelled the operation.

**************************************************************************/

bool ProgressDialog::Cancelled() const
{
	return m_bCancelled;
}

/**************************************************************************

>	void ProgressDialog::UpdateProgressDisplay(int PercentComplete)

	Author:		Tim Browse
	Date:		2001/10/27
	Inputs:		PercentComplete - new position of progress slider; must
								  be at least 0, and no more than 100.

	Purpose:	For use by ProgressWorker_OneShot objects to update
				the progress display.

**************************************************************************/

void ProgressDialog::UpdateProgressDisplay(int PercentComplete)
{
	// Update the progress bar, force the window to be updated, and service
	// the UI.
	SetPosition(PercentComplete);
	UpdateWindow();
	vgui::GetApp()->ProcessMessages();
}

/**************************************************************************

>	void ProgressDialog::SetPosition(int PercentComplete)

	Author:		Tim Browse
	Date:		2001/10/27
	Inputs:		PercentComplete - new position of progress slider; must
								  be at least 0, and no more than 100.

	Purpose:	Central function for setting the slider position and
				doing range-checking.

**************************************************************************/

void ProgressDialog::SetPosition(int PercentComplete)
{
	// Range sanity checks
	if (PercentComplete < 0)
	{
		PercentComplete = 0;
		COMPLAIN("Percentage value passed to ProgressDialog out of range!");
	}

	if (PercentComplete > 100)
	{
		PercentComplete = 100;
		COMPLAIN("Percentage value passed to ProgressDialog out of range!");
	}

	// Ok, update the progress bar position
	m_ProgressBar.SetPosition(PercentComplete);
}

