//
// SearchResultsList.cpp - a list control to display search results
//

#include "StdHdr.h"

#include "SearchResultsList.h"

#include "FileIndex.h"

#include "BinaryFileIndex.h"

#include "Res\Resource.h"

#include "vgui\WaitCursor.h"

#include "vos\Trace.h"
#include "vos\StringUtil.h"

//
// Redraw metrics...
//
static const int kIconSize = 16;
static const int kIconGap = 2;

//
// Column names
//

static const int kColumn_Name = 0;
static const int kColumn_Folder = 1;
static const int kColumn_Size = 2;
static const int kColumn_Date = 3;

/**************************************************************************

>	SearchResultsList::SearchResultsList()

	Author:		Tim Browse
	Date:		29/04/2001

	Purpose:	Constructor

**************************************************************************/

SearchResultsList::SearchResultsList()
{
	vgui::IconResourceID rFolderIcon(IDI_FOLDER);
	rFolderIcon.Load(&m_FolderIcon);
	vgui::IconResourceID rFileIcon(IDI_FILE);
	rFileIcon.Load(&m_FileIcon);
}

void SearchResultsList::Attach(vgui::Window* pParent, vgui::ControlID ID)
{
	// Get base class to attach the control to this object.
	ListBox::Attach(pParent, ID);

	// Now set up the control
	SetNumberOfColumns(4);
	SetColumnName(kColumn_Name, "Name");
	SetColumnWidth(kColumn_Name, 100);
	SetColumnName(kColumn_Folder, "Folder");
	SetColumnWidth(kColumn_Folder, 200);
	SetColumnName(kColumn_Size, "Size");
	SetColumnWidth(kColumn_Size, 60);
	SetColumnName(kColumn_Date, "Date");
	SetColumnWidth(kColumn_Date, 60);
}


/**************************************************************************

>	void SearchResultsList::ClearContents()

	Author:		Tim Browse
	Date:		29/04/2001

	Purpose:	Clear the list of all data.

**************************************************************************/

void SearchResultsList::ClearContents()
{
	Clear();
}


/**************************************************************************

>	void SearchResultsList::SetContents(FileIndex* pIndex)

	Author:		Tim Browse
	Date:		29/04/2001
	Inputs:		pIndex - the index to display the search results from.

	Purpose:	Set up the list with a list of search results.

**************************************************************************/

void SearchResultsList::SetContents(FileIndex* pIndex)
{
	// Remember this index
	m_pIndex = pIndex;

	// Hourglass while we do this
	vgui::WaitCursor MyWaitCursor;

	// Freeze window update so we get a smooth transition
	LockRedraw();

	// First, clear the list contents
	Clear();

	// Use a dummy string - we build display strings on demand for efficiency
	// reasons
	vos::String Name;

	// Now add an item for each result
	int cResults = m_pIndex->GetNumberOfSearchResults();

	int iItem;

	for (int iResult = 0; iResult < cResults; iResult++)
	{
		// Add it to the list
		Name = m_pIndex->GetLeafname(iResult);
		AddItem(Name, &iItem, (vgui::UserData) iResult);

		// Set folder name in column 1
		Name = m_pIndex->GetFolder(iResult);
		SetItemText(iItem, Name, kColumn_Folder);

		// Set file size in column 2 (for files)
		if (!m_pIndex->SearchResultIsFolder(iResult))
		{
			SetItemText(iItem, vos::StringUtil::GetStorageSize(m_pIndex->GetFileSize(iResult)), kColumn_Size);
		}

		// Display date/time info
		Name = m_pIndex->GetTimeStamp(iResult).FormatLocalTime("%d/%m/%y %H:%M");
		SetItemText(iItem, Name, kColumn_Date);
	}

	// Set up column sort functions
	SetSortFunction(CompareFileSizes, (vgui::UserData) m_pIndex, kColumn_Size);
	SetSortFunction(CompareTimestamps, (vgui::UserData) m_pIndex, kColumn_Date);

	// Make sure size/date columns are large enough
	AutoSizeColumn(kColumn_Size);
	AutoSizeColumn(kColumn_Date);

	// Ok, redraw the new stuff
	UnlockRedraw();
}

/**************************************************************************

>	bool SearchResultsList::GetSelectedFolder(vos::FileSpec* pfsFolder)

	Author:		Tim Browse
	Date:		2001/10/28
	Outputs:	pfsFolder - the name of the folder that contains the
							currently selected item.
	Returns:	true => folder found ok;
				false => there is no currently selected item

	Purpose:	Get the folder that contains the currently selected item
				in the search results.

**************************************************************************/

bool SearchResultsList::GetSelectedFolder(vos::FileSpec* pfsFolder)
{
	int iItem;
	if (GetSelectedItem(&iItem))
	{
		// Get the folder containing the selected item - we look
		// up the search result index for this item by reading
		// the userdata (because the listbox is sorted, the order we
		// add the items in is not the order they appear in).
		int iSearchIndex;
		if (GetItemUserData(iItem, (vgui::UserData*) &iSearchIndex))
		{
			vos::String strFolder = m_pIndex->GetFolder(iSearchIndex);

			// Return to caller
			pfsFolder->SetFromString(strFolder);
			return true;
		}
	}
	else
	{
		// No selected item
	}

	// Unable to get the selected folder
	return false;
}

#if 0

/**************************************************************************

>	bool SearchResultsList::OnDrawItem(vgui::GC* pGC, 
									   int iItem,
									   vgui::UserData TheUserData,
									   const vgui::Rect& rcItem,
									   int OwnerDrawItemState)

	Author:		Tim Browse
	Date:		29/04/2001
	Inputs:		See OwnerDrawListControl
	Returns:	See OwnerDrawListControl

	Purpose:	Draw the search result item

**************************************************************************/

bool SearchResultsList::OnDrawItem(vgui::GC* pGC, 
								   int iItem,
								   vgui::UserData TheUserData,
								   const vgui::Rect& rcItem,
								   int OwnerDrawItemState)
{
	if (iItem < 0)
		return false;

	// Get the search result for this list item.
	BinaryIndexEntry *pEntry = m_pIndex->GetSearchResult((int) TheUserData);

	if (pEntry == NULL)
	{
		COMPLAIN("Error drawing search results list - cannot find result entry.");
		return false;
	}

	// Windows-related bodge - we must save the GC and restore if DrawFocusRect()
	// is to work properly :-(
	pGC->Save();

	// Colours to use
	vgui::Colour FillColour;
	vgui::Colour TextColour;

	if (OwnerDrawItemState & vgui::OwnerDrawItemState_Selected)
	{
		// Use highlighted colours
		FillColour = vgui::Colour::GetGUIColour(vgui::Colour_SelectedBackground);
		TextColour = vgui::Colour::GetGUIColour(vgui::Colour_SelectedText);
	}
	else
	{
		// Use normal colours
		FillColour = vgui::Colour::GetGUIColour(vgui::Colour_WindowBackground);
		TextColour = vgui::Colour::GetGUIColour(vgui::Colour_WindowText);
	}

	// Instantiate these colours in the GC
	pGC->SetFillColour(FillColour);
	pGC->SetTextColour(TextColour);

	// Get the full pathname of this entry
	vos::String Text = m_pIndex->GetFullPathname((int) TheUserData);

	// Fill the item rectangle, and draw the pathname text
	pGC->DrawRect(rcItem, vgui::GC::dt_Fill);

	// Draw item text, moving it to the right according to size of the icons.
	vgui::Rect rcText = rcItem;
	rcText.left += (kIconGap + kIconSize + kIconGap);
	pGC->DrawText(Text, rcText, vgui::tf_Left|vgui::tf_VCentre);

	// Choose the icon to use - file or folder...
	vgui::Icon* pIcon = &m_FileIcon;
	
	if (pEntry->m_Type == IndexEntryType_Folder)
		pIcon = &m_FolderIcon;

	// Draw an icon for this entry
	pGC->DrawIcon(*pIcon, rcItem.left + kIconGap, rcItem.top + kIconGap);

	// If the item is selected, then draw the input focus rectangle around it.
	pGC->Restore();

	// If we have focus, draw focus rectangle on selected item.
	//
	// TODO: Enhance vgui OnDrawItem caller, so that it will draw the focus
	//		 rectangle on demand if the client wants it to.
	if ((OwnerDrawItemState & vgui::OwnerDrawItemState_Selected) && HasFocus())
	{
		pGC->DrawFocusRect(rcItem);
	}

	// drawing handled.
	return true;
}

/**************************************************************************

>	bool SearchResultsList::OnMeasureItem(int iItem, 
										  vgui::UserData TheUserData, 
										  uint* pItemHeight)

	Author:		Tim Browse
	Date:		2001/10/25
	Inputs:		iItem - index of the item to measure
				TheUserData - whatever data has been attached to the item.
	Outputs:	pItemHeight - the desired height of the item.
	Returns:	true => custom size was specified
				false => no size set up - caller should use default size.

	Purpose:	Find the height of the listitems.

**************************************************************************/

bool SearchResultsList::OnMeasureItem(int iItem, 
									  vgui::UserData TheUserData, 
									  uint* pItemHeight)
{
	*pItemHeight = kIconGap + kIconSize + kIconGap;
	return true;
}

#endif

int SearchResultsList::CompareFileSizes(const vgui::ListControl* pListControl, 
									    vgui::UserData TheUserData,
									    int iItem1, int iItem2, 
									    int iColumn)
{
	// Get the file index object
	FileIndex* pIndex = (FileIndex*) TheUserData;

	if (pIndex != NULL)
	{
		// Get the search result index
		int iSearchResult1,
			iSearchResult2;

		if (pListControl->GetItemUserData(iItem1, (vgui::UserData*) &iSearchResult1) &&
			pListControl->GetItemUserData(iItem2, (vgui::UserData*) &iSearchResult2))
		{
			int FileSize1 = pIndex->GetFileSize(iSearchResult1);
			int FileSize2 = pIndex->GetFileSize(iSearchResult2);

			if (FileSize1 < FileSize2)
				return -1;
			else if (FileSize1 > FileSize2)
				return 1;
			else return 0;
		}
	}

	// Can't get info - assume equivalence
	COMPLAIN("Unable to access item information when sorting!");
	return 0;
}

int SearchResultsList::CompareTimestamps(const vgui::ListControl* pListControl, 
									     vgui::UserData TheUserData,
									     int iItem1, int iItem2, 
									     int iColumn)
{
	// Get the file index object
	FileIndex* pIndex = (FileIndex*) TheUserData;

	if (pIndex != NULL)
	{
		// Get the search result index
		int iSearchResult1,
			iSearchResult2;

		if (pListControl->GetItemUserData(iItem1, (vgui::UserData*) &iSearchResult1) &&
			pListControl->GetItemUserData(iItem2, (vgui::UserData*) &iSearchResult2))
		{
			vos::Time FileTime1 = pIndex->GetTimeStamp(iSearchResult1);
			vos::Time FileTime2 = pIndex->GetTimeStamp(iSearchResult2);

			if (FileTime1 < FileTime2)
				return -1;
			else if (FileTime1 > FileTime2)
				return 1;
			else return 0;
		}
	}

	// Can't get info - assume equivalence
	COMPLAIN("Unable to access item information when sorting!");
	return 0;
}